import os

from boj.errors.error import FileIOError

def mkdir(path: str, exist_ok=True):
    try:
        os.makedirs(name=path, exist_ok=exist_ok)
    except OSError as e:
        raise e

def file_exists(path: str):
    if os.path.isfile(path):
        return True
    return False

class GeneralFileIO:
    def read(self, path: str) -> bytes:
        if not os.path.isfile(path):
            raise FileNotFoundError(f"'{path}' is not a file or does not exist")

        try:
            with open(path, "rb") as file:
                data = file.read()

            return data
        except Exception:
            raise FileIOError(f"Error while reading the file '{path}'")

    def write(self, raw: bytes, path: str) -> None:
        mkdir(os.path.dirname(path))
        with open(path, "wb") as file:
            file.write(raw)


import os
import time
import subprocess
from concurrent.futures import ThreadPoolExecutor, as_completed
from rich.console import Console
from bs4 import BeautifulSoup
import requests

from boj.core.solved import solvedAPI

console = Console()

class ProblemFetcher:
    def __init__(self, problem_id):
        self.problem_id = problem_id
        self.input_list = []
        self.output_list = []

    def fetch(self):
        problem_info = solvedAPI.fetch_problem_info(self.problem_id)
        level_title, level_num = solvedAPI.parse_problem_info(problem_info)[2]

        base_path = f"{level_title}/{level_num}/{self.problem_id}"
        page_content = self._fetch_problem_page()  # Fetch the problem page content
        soup = BeautifulSoup(page_content, "html.parser")
        self.input_list, self.output_list = self._parse_sample_data(soup, base_path)

    def _fetch_problem_page(self):
        url = f"https://www.acmicpc.net/problem/{self.problem_id}"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3"
        }
        response = requests.get(url, headers=headers)

        if response.status_code != 200:
            raise Exception(f"Error fetching problem page: {response.status_code}")

        return response.text

    def _parse_sample_data(self, soup, base_path):
        sample_data = soup.find_all(class_="sampledata")
        input_list = []
        output_list = []

        # Check if sample_data is empty or not
        if not sample_data:
            raise Exception("Sample data not found. The HTML structure might have changed.")

        os.makedirs(f"./{base_path}/data", exist_ok=True)
        for elem in sample_data:
            index = elem.get("id")[-1]
            content = elem.get_text(strip=True)

            if "input" in elem.get("id"):
                input_list.append(index)
                self._write_to_file(f"./{base_path}/data/{index}.in", content)
            elif "output" in elem.get("id"):
                output_list.append(index)
                self._write_to_file(f"./{base_path}/data/{index}.out", content)

        return input_list, output_list

    def _write_to_file(self, filepath, content):
        with open(filepath, "w", encoding="utf-8") as f:
            f.write(content)


class TestRunner:
    def __init__(self, problem_id, input_list, output_list):
        problem_info = solvedAPI.fetch_problem_info(problem_id)
        level_title, level_num = solvedAPI.parse_problem_info(problem_info)[2]

        self.problem_id = problem_id
        self.base_path = f"{level_title}/{level_num}/{self.problem_id}"
        self.input_list = input_list
        self.output_list = output_list
        self.results = []

    def run_tests(self):
        with ThreadPoolExecutor() as executor:
            future_to_index = {
                executor.submit(self._execute_test_case, index): index
                for index in self.input_list
            }
            for future in as_completed(future_to_index):
                index = future_to_index[future]
                self._update_console_status(future, index)

        self._evaluate_results()

    def _update_console_status(self, future, index):
        with console.status("Loading inputs...") as status:
            status.update(f"Test Case #{index} : [bold green]RUNNING[/]")
            try:
                result = future.result()
                self.results.append((index, result))
            except Exception as e:
                console.print(f"[bold red]Error in test Case #{index}: {e}[/bold red]")
                e.with_traceback("tb")
            time.sleep(1)

    def _evaluate_results(self):
        corrects = []

        for index, (actual_output, expected_output, input_value) in self.results:
            actual_output_lines = actual_output.strip().splitlines()
            expected_output_lines = expected_output.strip().splitlines()

            if actual_output_lines == expected_output_lines:
                self._print_result(index, actual_output, expected_output, input_value, "green")
                corrects.append(True)
            else:
                self._print_result(index, actual_output, expected_output, input_value, "red")
                corrects.append(False)

        self._print_summary(corrects)

    def _print_result(self, index, actual_output, expected_output, input_value, color):
        console.print(f"[bold {color}]---------------------------------------------------[/bold {color}]")
        console.print(f"[bold deep_sky_blue1]Test Case #{index} :[/bold deep_sky_blue1] [bold {color}]OUTPUT[/bold {color}]")
        console.print(f"[bold magenta]Input:[/bold magenta] \n{input_value.strip()}\n")
        console.print(f"[bold magenta]Expected:[/bold magenta] \n{expected_output.strip()}")
        console.print(f"[bold magenta]Yours:[/bold magenta] \n{actual_output.strip()}")

    def _print_summary(self, corrects):
        final_color = "green" if all(corrects) else "red"
        console.print(f"[bold {final_color}]---------------------------------------------------[/]")
        console.print("\n[bold green]All tests completed![/bold green]")

        for count, correct in enumerate(corrects, start=1):
            status = "PASS" if correct else "FAILED"
            color = "green" if correct else "red"
            console.print(f"[bold deep_sky_blue1]- Test Case #{count} :[/] [bold {color}]{status}[/]")

    def _execute_test_case(self, index):
        input_file = f"./{self.base_path}/data/{index}.in"
        expected_output_file = f"./{self.base_path}/data/{index}.out"

        actual_output = self._execute_solution(input_file)
        expected_output = self._read_file(expected_output_file)
        input_value = self._read_file(input_file)

        return actual_output, expected_output, input_value

    def _execute_solution(self, input_file):
        # Determine the file type and set the command accordingly
        solution_files = [f for f in os.listdir(f"./{self.base_path}") if os.path.isfile(os.path.join(f"./{self.base_path}", f))]
        solution_file = None

        for file in solution_files:
            if any(file.endswith(ext) for ext in [".py", ".cpp", ".c", ".go", ".cs", ".js", ".ts"]):
                solution_file = file
                break

        if not solution_file:
            raise Exception("No solution file found!")

        file_ext = os.path.splitext(solution_file)[1]

        command = []
        if file_ext == ".py":
            command = ["python", f"./{self.base_path}/{solution_file}"]
        elif file_ext == ".cpp":
            executable = f"./{self.base_path}/{solution_file.replace('.cpp', '')}"
            subprocess.run(["g++", f"./{self.base_path}/{solution_file}", "-o", executable, "-std=c++17"])
            command = [executable]
        elif file_ext == ".c":
            executable = f"./{self.base_path}/{solution_file.replace('.c', '')}"
            subprocess.run(["gcc", f"./{self.base_path}/{solution_file}", "-o", executable, "-std=c99"])
            command = [executable]
        elif file_ext == ".go":
            command = ["go", "run", f"./{self.base_path}/{solution_file}"]
        elif file_ext == ".cs":
            executable = f"./{self.base_path}/{solution_file.replace('.cs', '.exe')}"
            subprocess.run(["csc", "-out:" + executable, f"./{self.base_path}/{solution_file}"])
            command = [executable]
        elif file_ext == ".js":
            command = ["node", f"./{self.base_path}/{solution_file}"]
        elif file_ext == ".ts":
            js_output_file = f"./{self.base_path}/{solution_file.replace('.ts', '.js')}"
            subprocess.run(["tsc", f"./{self.base_path}/{solution_file}"])
            command = ["node", js_output_file]

        process = subprocess.Popen(
            command,
            stdin=open(input_file, "r"),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        stdout, _ = process.communicate()
        return stdout.decode("utf-8").strip()

    def _read_file(self, filepath):
        with open(filepath, "r", encoding="utf-8") as f:
            return f.read().strip()


class TestCaseExecutor:
    def __init__(self, problem_id):
        self.problem_id = problem_id

    def execute(self):
        fetcher = ProblemFetcher(self.problem_id)
        fetcher.fetch()
        runner = TestRunner(self.problem_id, fetcher.input_list, fetcher.output_list)
        runner.run_tests()


if __name__ == "__main__":
    solvedAPI.load_rank_info()
    executor = TestCaseExecutor(1000)
    executor.execute()



import argparse

from boj.commands.add_cmd import new_problem
from boj.commands.test_cmd import TestCaseExecutor
from boj.core.solved import solvedAPI


def cli():
    parser = argparse.ArgumentParser(description="CLI for managing problems.")
    subparsers = parser.add_subparsers(dest="command")

    # 'add' 명령어
    add_parser = subparsers.add_parser("add", help="Add a new problem")
    add_parser.add_argument("problem_id", type=int, help="The ID of the problem")
    add_parser.add_argument(
        "-l",
        "--lang",
        type=str,
        help="Programming language (e.g., C++17)",
        default="PyPy3",
    )

    test_parser = subparsers.add_parser("test", help="test a problem")
    test_parser.add_argument("problem_id", type=int, help="The ID of the problem")
    test_parser.add_argument(
        "-l",
        "--lang",
        type=str,
        help="Programming language (e.g., C++17)",
        default="PyPy3",
    )

    args = parser.parse_args()

    if args.command == "add":
        new_problem(args.problem_id, args.lang)
    if args.command == "test":
        solvedAPI.load_rank_info()
        executor = TestCaseExecutor(1000)
        executor.execute()




from typing import Optional

import time
import random
from rich.console import Console, RenderableType
from rich.status import Status
from rich.style import StyleType
from rich import print

class BojConsole(Console):
    error_color = "red"
    info_color = "blue"
    warn_color = "yellow"

    def __init__(self):
        super().__init__(log_time=False, log_path=False)

    def error(self, message):
        super().print(f"[bold {self.error_color}]Error: {message}[/]")

    def info(self, message):
        super().print(f"[bold {self.info_color}]Info: {message}[/]")

    def warn(self, message):
        super().print(f"[bold {self.warn_color}]Warning: {message}[/]")
    
    def status(
        self,
        status: RenderableType,
        *,
        spinner: str = "dots",
        spinner_style: str = "status.spinner",
        speed: float = 1.0,
        refresh_per_second: float = 12.5,
    ) -> Status:
        status_renderable = BojStatus(
            f"{status}",
            console=self,
            spinner=spinner,
            spinner_style="white",
            speed=speed,
            refresh_per_second=refresh_per_second,
        )
        return status_renderable


class BojStatus(Status):
    status_color: str = "[bold blue]"

    def __init__(
        self,
        status: RenderableType,
        *,
        console: Optional[Console] = None,
        spinner: str = "dots",
        spinner_style: StyleType = "status.spinner",
        speed: float = 1.0,
        refresh_per_second: float = 12.5,
    ):
        super().__init__(
            status=status,
            console=console,
            spinner=spinner,
            spinner_style=spinner_style,
            speed=speed,
            refresh_per_second=refresh_per_second,
        )

    def update(
        self,
        status: Optional[RenderableType] = None,
        *,
        spinner: Optional[str] = None,
        spinner_style: Optional[StyleType] = None,
        speed: Optional[float] = None,
    ) -> None:
        time.sleep(0.08 + random.uniform(0.1, 0.2))
        super().update(status=f"{self.status_color}{status}[/]")
        time.sleep(0.08 + random.uniform(0.1, 0.2))




import time
from typing import Dict, Tuple
import requests
import json
from rich.console import Console

console = Console()

class solvedAPI:
    rank_info: Dict[int, Tuple[str, str]] = {}

    @staticmethod
    def load_rank_info():
        with console.status("Load level.json...") as status:
            time.sleep(0.1)
        with open("boj/settings/level.json", "r", encoding="utf-8") as f:
            data = json.load(f)
            solvedAPI.rank_info = {int(k): (v[0], v[1]) for k, v in data.items()}

    @staticmethod
    def fetch_problem_info(id: int):
        url = "https://solved.ac/api/v3/problem/show"

        querystring = {"problemId": f"{id}"}

        headers = {"x-solvedac-language": "ko", "Accept": "application/json"}
        with console.status("Get problem info...") as status:
            time.sleep(0.1)
            status.update(f"Get problem info from {url} with {querystring}, {headers}")
        response = requests.get(url, headers=headers, params=querystring)

        return response.json()

    @staticmethod
    def get_level(level):
        with console.status("Get level info...") as status:
            time.sleep(0.1)
        return solvedAPI.rank_info[level]

    @staticmethod
    def parse_problem_info(response):
        with console.status("Parsing problem info...") as status:
            time.sleep(0.1)
        return (
            response["problemId"],
            response["titleKo"],
            solvedAPI.get_level(response["level"]),
        )



import json
import os
import time
from rich import print
from rich.console import Console

from boj.core.solved import solvedAPI

console = Console()

def create_level_folder(problem_id: int, rank: list[str, str], lang="python") -> None:
    created = False

    level_part, level_num = rank
    directory_path = os.path.join(level_part, level_num, str(problem_id))
    os.makedirs(directory_path, exist_ok=True)
    with open("boj/settings/language.json", "r", encoding="utf-8") as f:
        data = json.load(f)
        extension = data[lang]
    file_path = os.path.join(directory_path, f"main{extension}")
    with console.status("Checking file existence...") as status:
        time.sleep(0.1)
        if not os.path.exists(file_path):
            status.update("No file exists. Creating file...")
            open(file_path, "w", encoding="utf-8").close()
            created = True
            time.sleep(0.1)
        else:
            status.update(f"{file_path} already exists.")
            time.sleep(0.1)

    if created:
        print(f"{file_path} successfully created!")
    elif not created:
        print(f"successfully ended.")


def new_problem(id, lang="PyPy3"):
    try:
        solvedAPI.load_rank_info()
        response = solvedAPI.fetch_problem_info(id)

        problem_id, _, level = solvedAPI.parse_problem_info(response)
        create_level_folder(problem_id, level, lang)
    except Exception as e:
        print(f"[bold red]Error : {e}[/bold red]")
